@@ -1254,9 +1254,9 @@ var _ = Describe("Subscription", func() {
 		require.Len(GinkgoT(), installPlan.Status.CatalogSources, 1)
 	})
 
-	// CatSrc:
+	// CatSrc1:
 	//
-	// Package A (apackage)
+	// Package 1 (apackage)
 	// Default Channel: Stable
 	// Channel Stable:
 	// Operator A (Requires: CRD 1, CRD 2 )
@@ -1266,12 +1266,19 @@ var _ = Describe("Subscription", func() {
 	// Default Channel: Stable
 	// Channel Stable:
 	// Operator B (Provides: CRD)
-	// CatSrc2:
+	// Channel Alpha:
+	// Operator D (Provides: CRD)
 	//
-	// Package B (bpackage)
+	// CatSrc2:
+	// Package 2 (bpackage)
 	// Default Channel: Stable
 	// Channel Stable:
 	// Operator C (Provides: CRD 2)
+	// Package 3 (cpackage)
+	// Default Channel: Stable
+	// Channel Stable:
+	// Operator E (Provides: CRD 2)
+	//
 	// Then create a subscription:
 	//
 	// CatalogSource: CatSrc
@@ -1280,10 +1287,11 @@ var _ = Describe("Subscription", func() {
 	// StartingCSV: CSV A
 	//
 	// Check installed:
+	// CSV A, CSV B, CSV E
 	//
-	// CSV A, CSV B, CSV C
-	//
-	// CSV A required B and C but didn't get them from Package A
+	// CSV ABC: not chosen as it is the same package with CSV A
+	// CSV D: not chosen as it is in non-default channel
+	// CSV C: not chosen as it is the same package with CSV B (which is chosen)
 	It("creation with dependencies required and provided in different versions of an operator in the same package", func() {
 
 		defer cleaner.NotifyTestComplete(true)
@@ -1335,6 +1343,7 @@ var _ = Describe("Subscription", func() {
 		// Create CSV
 		packageName1 := genName("apackage")
 		packageName2 := genName("bpackage")
+		packageName3 := genName("cpackage")
 
 		namedStrategy := newNginxInstallStrategy((genName("dep")), permissions, nil)
 		depNamedStrategy := newNginxInstallStrategy((genName("dep")), permissions, nil)
@@ -1350,6 +1359,8 @@ var _ = Describe("Subscription", func() {
 		csvC := newCSV("nginx-c-dep", testNamespace, "", semver.MustParse("0.1.0"), []apiextensions.CustomResourceDefinition{crd2}, nil, depNamedStrategy2)
 		// csvD provides CRD1 in the same catalogsource with csvA (apackage)
 		csvD := newCSV("nginx-d-dep", testNamespace, "", semver.MustParse("0.1.0"), []apiextensions.CustomResourceDefinition{crd}, nil, depNamedStrategy)
+		// csvE provides CRD2 in the different catalogsource with csvC (bpackage)
+		csvE := newCSV("nginx-e-dep", testNamespace, "", semver.MustParse("0.1.0"), []apiextensions.CustomResourceDefinition{crd2}, nil, depNamedStrategy2)
 
 		// Create PackageManifests 1
 		// Contain csvA, ABC and B
@@ -1382,6 +1393,13 @@ var _ = Describe("Subscription", func() {
 				},
 				DefaultChannelName: stableChannel,
 			},
+			{
+				PackageName: packageName3,
+				Channels: []registry.PackageChannel{
+					{Name: stableChannel, CurrentCSVName: csvE.GetName()},
+				},
+				DefaultChannelName: stableChannel,
+			},
 		}
 
 		catalogSourceName := genName("catsrc")
@@ -1402,7 +1420,7 @@ var _ = Describe("Subscription", func() {
 		}
 
 		catalogSourceName2 := genName("catsrc")
-		catsrc2, cleanup2 := createInternalCatalogSource(kubeClient, crClient, catalogSourceName2, testNamespace, manifests2, []apiextensions.CustomResourceDefinition{crd2}, []v1alpha1.ClusterServiceVersion{csvC})
+		catsrc2, cleanup2 := createInternalCatalogSource(kubeClient, crClient, catalogSourceName2, testNamespace, manifests2, []apiextensions.CustomResourceDefinition{crd2}, []v1alpha1.ClusterServiceVersion{csvC, csvE})
 		defer cleanup2()
 
 		// Ensure that the catalog source is resolved before we create a subscription.
@@ -1424,7 +1442,7 @@ var _ = Describe("Subscription", func() {
 		// Fetch CSVs A, B and C
 		_, err = fetchCSV(GinkgoT(), crClient, csvB.Name, testNamespace, csvSucceededChecker)
 		require.NoError(GinkgoT(), err)
-		_, err = fetchCSV(GinkgoT(), crClient, csvC.Name, testNamespace, csvSucceededChecker)
+		_, err = fetchCSV(GinkgoT(), crClient, csvE.Name, testNamespace, csvSucceededChecker)
 		require.NoError(GinkgoT(), err)
 		_, err = fetchCSV(GinkgoT(), crClient, csvA.Name, testNamespace, csvSucceededChecker)
 		require.NoError(GinkgoT(), err)
@@ -1435,6 +1453,189 @@ var _ = Describe("Subscription", func() {
 		_, err = crClient.OperatorsV1alpha1().ClusterServiceVersions(testNamespace).Get(context.TODO(), csvD.Name, metav1.GetOptions{})
 		require.Error(GinkgoT(), err)
 	})
+
+	// csvA owns CRD1 & csvB owns CRD2 and requires CRD1
+	// Create subscription for csvB lead to installation of csvB and csvA
+	// Update catsrc to upgrade csvA to csvNewA which now requires CRD1
+	// csvNewA can't be installed due to no other operators provide CRD1 for it
+	// (Note: OLM can't pick csvA as dependency for csvNewA as it is from the same
+	// same package)
+	// Update catsrc again to upgrade csvB to csvNewB which now owns both CRD1 and
+	// CRD2.
+	// Now csvNewA and csvNewB are installed successfully as csvNewB provides CRD1
+	// that csvNewA requires
+	It("creation in case of transferring providedAPIs", func() {
+
+		kubeClient := newKubeClient()
+		crClient := newCRClient()
+
+		permissions := deploymentPermissions()
+
+		crdPlural := genName("ins")
+		crdName := crdPlural + ".cluster.com"
+		crdPlural2 := genName("ins")
+		crdName2 := crdPlural2 + ".cluster.com"
+
+		crd := apiextensions.CustomResourceDefinition{
+			ObjectMeta: metav1.ObjectMeta{
+				Name: crdName,
+			},
+			Spec: apiextensions.CustomResourceDefinitionSpec{
+				Group:   "cluster.com",
+				Version: "v1alpha1",
+				Names: apiextensions.CustomResourceDefinitionNames{
+					Plural:   crdPlural,
+					Singular: crdPlural,
+					Kind:     crdPlural,
+					ListKind: "list" + crdPlural,
+				},
+				Scope: "Namespaced",
+			},
+		}
+
+		crd2 := apiextensions.CustomResourceDefinition{
+			ObjectMeta: metav1.ObjectMeta{
+				Name: crdName2,
+			},
+			Spec: apiextensions.CustomResourceDefinitionSpec{
+				Group:   "cluster.com",
+				Version: "v1alpha1",
+				Names: apiextensions.CustomResourceDefinitionNames{
+					Plural:   crdPlural2,
+					Singular: crdPlural2,
+					Kind:     crdPlural2,
+					ListKind: "list" + crdPlural2,
+				},
+				Scope: "Namespaced",
+			},
+		}
+
+		// Create CSV
+		packageName1 := genName("apackage")
+		packageName2 := genName("bpackage")
+
+		namedStrategy := newNginxInstallStrategy((genName("dep")), permissions, nil)
+		namedStrategy2 := newNginxInstallStrategy((genName("dep")), permissions, nil)
+		// csvA provides CRD
+		csvA := newCSV("nginx-a", testNamespace, "", semver.MustParse("0.1.0"), []apiextensions.CustomResourceDefinition{crd}, nil, namedStrategy)
+		// csvB provides CRD2 and requires CRD
+		csvB := newCSV("nginx-b", testNamespace, "", semver.MustParse("0.1.0"), []apiextensions.CustomResourceDefinition{crd2}, []apiextensions.CustomResourceDefinition{crd}, namedStrategy2)
+		// New csvA requires CRD (transfer CRD ownership to the new csvB)
+		csvNewA := newCSV("nginx-new-a", testNamespace, "nginx-a", semver.MustParse("0.2.0"), nil, []apiextensions.CustomResourceDefinition{crd}, namedStrategy)
+		// New csvB provides CRD and CRD2
+		csvNewB := newCSV("nginx-new-b", testNamespace, "nginx-b", semver.MustParse("0.2.0"), []apiextensions.CustomResourceDefinition{crd, crd2}, nil, namedStrategy2)
+
+		// Create PackageManifests 1
+		// Contain csvA, ABC and B
+		manifests := []registry.PackageManifest{
+			{
+				PackageName: packageName1,
+				Channels: []registry.PackageChannel{
+					{Name: stableChannel, CurrentCSVName: csvA.GetName()},
+				},
+				DefaultChannelName: stableChannel,
+			},
+			{
+				PackageName: packageName2,
+				Channels: []registry.PackageChannel{
+					{Name: stableChannel, CurrentCSVName: csvB.GetName()},
+				},
+				DefaultChannelName: stableChannel,
+			},
+		}
+
+		catalogSourceName := genName("catsrc")
+		catsrc, cleanup := createInternalCatalogSource(kubeClient, crClient, catalogSourceName, testNamespace, manifests, []apiextensions.CustomResourceDefinition{crd, crd2}, []v1alpha1.ClusterServiceVersion{csvA, csvB})
+		defer cleanup()
+
+		// Ensure that the catalog source is resolved before we create a subscription.
+		_, err := fetchCatalogSourceOnStatus(crClient, catsrc.GetName(), testNamespace, catalogSourceRegistryPodSynced)
+		require.NoError(GinkgoT(), err)
+
+		subscriptionSpec := &v1alpha1.SubscriptionSpec{
+			CatalogSource:          catsrc.GetName(),
+			CatalogSourceNamespace: catsrc.GetNamespace(),
+			Package:                packageName2,
+			Channel:                stableChannel,
+			StartingCSV:            csvB.GetName(),
+			InstallPlanApproval:    v1alpha1.ApprovalAutomatic,
+		}
+
+		// Create a subscription that has a dependency
+		subscriptionName := genName("sub-")
+		cleanupSubscription := createSubscriptionForCatalogWithSpec(GinkgoT(), crClient, testNamespace, subscriptionName, subscriptionSpec)
+		defer cleanupSubscription()
+
+		subscription, err := fetchSubscription(crClient, testNamespace, subscriptionName, subscriptionStateAtLatestChecker)
+		require.NoError(GinkgoT(), err)
+		require.NotNil(GinkgoT(), subscription)
+
+		// Check that a single catalog source was used to resolve the InstallPlan
+		_, err = fetchInstallPlan(GinkgoT(), crClient, subscription.Status.InstallPlanRef.Name, buildInstallPlanPhaseCheckFunc(v1alpha1.InstallPlanPhaseComplete))
+		require.NoError(GinkgoT(), err)
+		// Fetch CSVs A and B
+		_, err = fetchCSV(GinkgoT(), crClient, csvA.Name, testNamespace, csvSucceededChecker)
+		require.NoError(GinkgoT(), err)
+		_, err = fetchCSV(GinkgoT(), crClient, csvB.Name, testNamespace, csvSucceededChecker)
+		require.NoError(GinkgoT(), err)
+
+		// Update PackageManifest
+		manifests = []registry.PackageManifest{
+			{
+				PackageName: packageName1,
+				Channels: []registry.PackageChannel{
+					{Name: stableChannel, CurrentCSVName: csvNewA.GetName()},
+				},
+				DefaultChannelName: stableChannel,
+			},
+			{
+				PackageName: packageName2,
+				Channels: []registry.PackageChannel{
+					{Name: stableChannel, CurrentCSVName: csvB.GetName()},
+				},
+				DefaultChannelName: stableChannel,
+			},
+		}
+		updateInternalCatalog(GinkgoT(), kubeClient, crClient, catalogSourceName, testNamespace, []apiextensions.CustomResourceDefinition{crd, crd2}, []v1alpha1.ClusterServiceVersion{csvNewA, csvA, csvB}, manifests)
+		csvAsub := strings.Join([]string{packageName1, stableChannel, catalogSourceName, testNamespace}, "-")
+		_, err = fetchSubscription(crClient, testNamespace, csvAsub, subscriptionStateUpgradeAvailableChecker)
+		require.NoError(GinkgoT(), err)
+		// Ensure csvNewA is not installed
+		_, err = crClient.OperatorsV1alpha1().ClusterServiceVersions(testNamespace).Get(context.Background(), csvNewA.Name, metav1.GetOptions{})
+		require.Error(GinkgoT(), err)
+		// Ensure csvA still exists
+		_, err = fetchCSV(GinkgoT(), crClient, csvA.Name, testNamespace, csvSucceededChecker)
+		require.NoError(GinkgoT(), err)
+
+		// Update packagemanifest again
+		manifests = []registry.PackageManifest{
+			{
+				PackageName: packageName1,
+				Channels: []registry.PackageChannel{
+					{Name: stableChannel, CurrentCSVName: csvNewA.GetName()},
+				},
+				DefaultChannelName: stableChannel,
+			},
+			{
+				PackageName: packageName2,
+				Channels: []registry.PackageChannel{
+					{Name: stableChannel, CurrentCSVName: csvNewB.GetName()},
+				},
+				DefaultChannelName: stableChannel,
+			},
+		}
+		updateInternalCatalog(GinkgoT(), kubeClient, crClient, catalogSourceName, testNamespace, []apiextensions.CustomResourceDefinition{crd, crd2}, []v1alpha1.ClusterServiceVersion{csvA, csvB, csvNewA, csvNewB}, manifests)
+		subscription, err = fetchSubscription(crClient, testNamespace, subscriptionName, subscriptionHasInstallPlanDifferentChecker(subscription.Status.InstallPlanRef.Name))
+		require.NoError(GinkgoT(), err)
+		_, err = fetchInstallPlan(GinkgoT(), crClient, subscription.Status.InstallPlanRef.Name, buildInstallPlanPhaseCheckFunc(v1alpha1.InstallPlanPhaseComplete))
+		require.NoError(GinkgoT(), err)
+		// Ensure csvNewB is installed
+		_, err = fetchCSV(GinkgoT(), crClient, csvNewB.Name, testNamespace, csvSucceededChecker)
+		require.NoError(GinkgoT(), err)
+		// Ensure csvNewA is installed
+		_, err = fetchCSV(GinkgoT(), crClient, csvNewA.Name, testNamespace, csvSucceededChecker)
+		require.NoError(GinkgoT(), err)
+	})
 })
 
 const (
